%% ------------------0-Add functions to the search path--------------------
addpath('../System');

%% ------------------1-Parameters of the two sections----------------------

tau_c = 12.5e-12 ;              % cavity single trip (i.e. 40 GHz rep rate)

% par is a vector containing the values of the 16 parameters (in s^-1)

% parameter for the gain section %

par1 = [
 6.0e10   ...   % 1---waveguide  response : internal losses        (s^-1)
  0.e11   ...   % 2---waveguide  response : real part of the dfb   (s^-1)
  0.e11   ...   % 3---waveguide  response : imag part of the dfb   (s^-1)
 3.0e11   ...   % 4----Material  response : sat gain/abs factor    (s^-1)
  8.e12   ...   % 5----Material  response : intraband relax time   (s^-1)
  0.e12   ...   % 6----Material  response : bandgap offset     (rad.s^-1)
 90.e12   ...   % 7----Material  response : Span of the band   (rad.s^-1)
  8.e11   ...   % 8----Material  response : elec thermal width (rad.s^-1)
  8.e12   ...   % 9----Material  response : hole thermal width (rad.s^-1)
  0.      ...   % 10---Material  response : intraband sat nrj   (unsused)
 150e08   ...   % 11----Carrier  response : injected carrier rate  (s^-1)
  1.e08   ...   % 12----Carrier  response : non rad decay rate     (s^-1)
  7.e08   ...   % 13----Carrier  response :     rad decay rate     (s^-1)
  1.e07   ...   % 14----Carrier  response :   Auger decay rate     (s^-1)
  1.e12   ...   % 15----Carrier  response :  Pop Grating diff rate (s^-1)  
  3.e-03  ...   % 16----Carrier  response : Sp emission rate        (a.u)                               
 ]';

% parameter for the absorber section %

par2 = [
 6.0e10   ...   % 1---waveguide  response : internal losses        (s^-1)
  0.e11   ...   % 2---waveguide  response : real part of the dfb   (s^-1)
  0.e11   ...   % 3---waveguide  response : imag part of the dfb   (s^-1)
 9.0e11   ...   % 4----Material  response : sat gain/abs factor    (s^-1)
  2.e12   ...   % 5----Material  response : intraband relax time   (s^-1)
  6.e12   ...   % 6----Material  response : bandgap offset     (rad.s^-1)
 90.e12   ...   % 7----Material  response : Span of the band   (rad.s^-1)
  8.e11   ...   % 8----Material  response : elec thermal width (rad.s^-1)
  8.e12   ...   % 9----Material  response : hole thermal width (rad.s^-1)
  0.      ...   % 10---Material  response : intraband sat nrj   (unsused)
  0.      ...   % 11----Carrier  response : injected carrier rate  (s^-1)
  1.e11   ...   % 12----Carrier  response : non rad decay rate     (s^-1)
  0.      ...   % 13----Carrier  response :     rad decay rate     (s^-1)
  0.      ...   % 14----Carrier  response :   Auger decay rate     (s^-1)
  1.e12   ...   % 15----Carrier  response :  Pop Grating diff rate (s^-1)  
  3.e-03  ...   % 16----Carrier  response : Sp emission rate        (a.u)                               
 ]';

% The boundary matrix must link the two outputs Em(0,1:2) and Ep(L,1:2) of 
% the two sections to the two inputs of the two sections Ep(0,1:2) and Em(L,1:2)
% Hence the boundary matrix is a 4x4 matrix 

BC = zeros(4,4)  ; 

BC(1,1) = sqrt(0.30) ;  %  Cleaved facet reflectivity -- 30 %
BC(2,3) = 1.         ;  %  Transmission of Em(0,2) to Em(L,1)
BC(3,2) = 1.         ;  %  Transmission of Ep(L,1) to Ep(0,2)
BC(4,4) = sqrt(0.30) ;  %  Cleaved facet reflectivity -- 30 %

BC = complex(BC) ;      %  The BC matrix must be complex 
                        
                     
%% ------------------2-Discretization and scaled parameter profiles--------                          

% N1 (N2) ; Number of point in space    for the first (second) section.                       
% M1 (M2) ; Number of point in the past for the first (second) section.
%      dt ; Time step used in the parameter scaling

s = 0.03 ; % Absorber length in percent 

[ M , N , dt ] = gnr_good_discr( [tau_c*( 1-s) tau_c*s] , [par1, par2] ) 

PAR1   = gnr_uni_param( N(1) , dt , par1 ) ; % An uniform scaled parameter set of N1 pts
PAR2   = gnr_uni_param( N(2) , dt , par2 ) ; % An uniform scaled parameter set of N2 pts

figure(1) ; clf ; gnr_chi_plot( 0.01 , M(2) , dt , par2(5:9) )  ; % the saturable aborption
figure(2) ; clf ; gnr_chi_plot( 3.0  , M(1) , dt , par1(5:9) )  ; % plot the gain as well as


%% ------------------3-Spatially uniform IC for the two sections-----------

% Generate an initial condition, i.e. a mesh of M points in the past
% and N points in space. All variables are set to zero excepted the carrier 
% that are set around 2.5 in the gain section and 0 in the absorber. 
% We add a gaussian fluctuation of 1.e-03. 

MSH1 = gnr_uni_msh( M(1) , N(1) , 1.e-01 , 3   )  ;
MSH2 = gnr_uni_msh( M(2) , N(2) , 1.e-01 , 0.01)  ;

%% ------------------4-Time integration -----------------------------------

% Time integration, over 100 single trips, of two sections (V=2), 
% saving data every dt (Samp=1) not using decimation for both sections (D1=D2=1)

S = 1000*( N(1)+N(2) ) ;  V = 2 ; Samp = 1 ; D1 = 1 ; D2 = 1 ; 

tic; [ tr1 , MSH1 , MSH2 ] = freetwm( S , V , Samp , MSH1 , PAR1 , D1 , MSH2 , PAR2 , D2 , BC ) ; toc


%% ------------------5-Data processing-------------------------------------

time = (1:S)*(Samp*dt)/1.e-09;   % create a time index in nanosecond

Em0 = tr1(:,1,1) ;                  % The output field on the left  of the first section  E_-(0)
EpL = tr1(:,2,2) ;                  % The output field on the right of the second section E_+(1)

D00 = tr1(:,3,1) ;                  % The carrier density on the left  of the first section  D_0(0)
D0L = tr1(:,4,2) ;                  % The carrier density on the right of the second section D_0(1)

fig=figure(3);clf;set( fig , 'Position' , [0 0 1600 750] ) ;

% ------------ Plot left  output intensity ------------- %

subplot(2,6,1) ; set(gca,'FontSize',22,'LineWidth',2);
plot ( time , abs(Em0).^2 , 'b' , 'LineWidth', 2) ;    
legend('I_-(0)','Location','Best') ; %xlabel('Time (ns)') ; 
grid on; axis([0 12.5 0 400]);

subplot(2,6,3) ; set(gca,'FontSize',22,'LineWidth',2);
plot ( time , abs(Em0).^2 , 'b' , 'LineWidth', 2) ;    
legend('I_-(0)','Location','Best') ; %xlabel('Time (ns)') ; 
grid on; axis([12 12.09 0 400]);

% ------------ Plot right output intensity ------------- %

subplot(2,6,2) ; set(gca,'FontSize',22,'LineWidth',2);
plot ( time , abs(EpL).^2 , 'r' , 'LineWidth', 2) ;  
legend('I_+(1)','Location','Best') ; %xlabel('Time (ns)') ; 
grid on ;  axis([0 12.5 0 400]);

subplot(2,6,4) ; set(gca,'FontSize',22,'LineWidth',2);
plot ( time , abs(EpL).^2 , 'r' , 'LineWidth', 2) ;  
legend('I_+(1)','Location','Best') ; %xlabel('Time (ns)') ; 
grid on ;  axis([12 12.09 0 400]);

% ------------ Plot left  carrier density  ------------- %

subplot(2,6,7) ; set(gca,'FontSize',22,'LineWidth',2);
plot ( time ,     D00     , 'k' , 'LineWidth', 2) ;  
legend('D_0(0)','Location','Best') ;  xlabel('Time (ns)') ; 
grid on ;  axis([0 12.5 2 4]);

subplot(2,6,9) ; set(gca,'FontSize',22,'LineWidth',2);
plot ( time ,     D00     , 'k' , 'LineWidth', 2) ;  
legend('D_0(0)','Location','Best') ;  xlabel('Time (ns)') ; 
grid on ;  axis([12 12.09 2 4]);

% ------------ Plot right  carrier density  ------------- %

subplot(2,6,8) ; set(gca,'FontSize',22,'LineWidth',2);
plot ( time ,     D0L     , 'k' , 'LineWidth', 2) ;  
legend('D_0(1)','Location','Best') ; xlabel('Time (ns)') ; 
grid on ; axis([0 12.5 0 1]);

subplot(2,6,10) ; set(gca,'FontSize',22,'LineWidth',2);
plot ( time ,     D0L     , 'k' , 'LineWidth', 2) ;  
legend('D_0(1)','Location','Best') ; xlabel('Time (ns)') ; 
grid on ; axis([12 12.09 0 1]);


% We perform a Optical spectrum of the field once 
% it reached steady state and keep only half of the points.

[ nu1 , specEp ] = OpticalSpectrum(     EpL(end/2:end)     , Samp*dt*1.e12 , 'normalize' ) ;  % step in ps
[ nu2 , specIp ] = OpticalSpectrum( abs(EpL(end/2:end)).^2 , Samp*dt*1.e09 , 'normalize' ) ;  % step in ns

% ------------ Plot the  optical spectrum ------------- %

subplot(2,6,[5 6]) ;  set(gca,'FontSize',22,'LineWidth',2);
semilogy(nu1,abs(specEp).^2) ; 
xlim([0 2]); 
ylim([1.e-10 1]); 
grid on;
xlabel('Frequency (THz)') ; 
legend('OS (dB)','Location','SouthEast') ; 

% ------------ Plot the RF spectrum  ------------------ %

subplot(2,6,[11 12]) ;  set(gca,'FontSize',22,'LineWidth',2);
semilogy(nu2,abs(specIp).^2, 'LineWidth', 2) ;   
xlim([0 50]); 
ylim([1.e-10 1]); 
grid on;
xlabel('Frequency (GHz)') ; 
legend('RF (dB)','Location','Best');

%------------------------------------------------------ %







