%% ------------------0-Add functions to the search path--------------------
addpath('../System');

%% ------------------1-Parameters of the two sections----------------------
  
tau_c=12.5e-12;   % cavity single trip (i.e. 40 GHz rep rate)

% par is a vector containing the values of the 16 parameters (in s^-1)

% parameter for the gain section % 75 cm^-1

par1= [
 6.0e10   ...   % 1---waveguide  response : internal losses        (s^-1)
  0.e11   ...   % 2---waveguide  response : real part of the dfb   (s^-1)
  0.e11   ...   % 3---waveguide  response : imag part of the dfb   (s^-1)
 3.0e11   ...   % 4----Material  response : sat gain/abs factor    (s^-1)
  8.e12   ...   % 5----Material  response : intraband relax time   (s^-1)
  0.e12   ...   % 6----Material  response : bandgap offset     (rad.s^-1)
120.e12   ...   % 7----Material  response : Span of the band   (rad.s^-1)
  8.e11   ...   % 8----Material  response : elec thermal width (rad.s^-1)
  8.e12   ...   % 9----Material  response : hole thermal width (rad.s^-1)
  0.      ...   % 10---Material  response : intraband sat nrj   ( unused)
 110e08   ...   % 11----Carrier  response : injected carrier rate  (s^-1)
  1.e08   ...   % 12----Carrier  response : non rad decay rate     (s^-1)
  7.e08   ...   % 13----Carrier  response :     rad decay rate     (s^-1)
  1.e07   ...   % 14----Carrier  response :   Auger decay rate     (s^-1)
  1.e12   ...   % 15----Carrier  response :  Pop Grating diff rate (s^-1)  
  1.e-06  ...   % 16----Carrier  response : Sp emission rate        (a.u)                               
 ]';
                              

% parameter for the absorber section % 300 cm^-1

par2= [
 6.0e10   ...   % 1---waveguide  response : internal losses        (s^-1)
  0.e11   ...   % 2---waveguide  response : real part of the dfb   (s^-1)
  0.e11   ...   % 3---waveguide  response : imag part of the dfb   (s^-1)
12.0e11   ...   % 4----Material  response : sat gain/abs factor    (s^-1)
  8.e12   ...   % 5----Material  response : intraband relax time   (s^-1)
  0.e12   ...   % 6----Material  response : bandgap offset     (rad.s^-1)
120.e12   ...   % 7----Material  response : Span of the band   (rad.s^-1)
  8.e11   ...   % 8----Material  response : elec thermal width (rad.s^-1)
  8.e12   ...   % 9----Material  response : hole thermal width (rad.s^-1)
  0.      ...   % 10---Material  response : intraband sat nrj   ( unused)
  0.      ...   % 11----Carrier  response : injected carrier rate  (s^-1)
  1.e11   ...   % 12----Carrier  response : non rad decay rate     (s^-1)
  0.      ...   % 13----Carrier  response :     rad decay rate     (s^-1)
  0.      ...   % 14----Carrier  response :   Auger decay rate     (s^-1)
  1.e12   ...   % 15----Carrier  response :  Pop Grating diff rate (s^-1)  
  1.e-06  ...   % 16----Carrier  response : Sp emission rate        (a.u)                               
 ]';

% The boundary matrix must link the two outputs Em(0,1:2) and Ep(L,1:2) of 
% the two sections to the two inputs of the two sections Ep(0,1:2) and Em(L,1:2)
% Hence the boundary matrix is a 4x4 matrix 

BC = zeros(4,4)  ; 

BC(1,1) = sqrt(0.30) ;  %  Cleaved facet reflectivity -- 30 %
BC(2,3) = 1.         ;  %  Transmission of Em(0,2) to Em(L,1)
BC(3,2) = 1.         ;  %  Transmission of Ep(L,1) to Ep(0,2)
BC(4,4) = sqrt(0.30) ;  %  Cleaved facet reflectivity -- 30 %

BC = complex(BC) ;      %  The BC matrix must be complex 
                        
                     
%% ------------------2-Discretization and scaled parameter profiles--------                          
                 
M1   =  33 ;   %  Number of point in the past for the first section.
M2   =  33 ;   %  Number of point in the past for the second section.

N1 = 512 + 1 ; %  Number of point in space for the first section is  2^P1+1    
N2 =  16 + 1 ; %  Number of point in space for the second section is 2^P2+1   

dt    = tau_c/(N1+N2)  ;                   % Time step used in the parameter scaling

PAR1   = gnr_uni_param( N1 , dt , par1 ) ; % An uniform scaled parameter set of N1 pts
PAR2   = gnr_uni_param( N2 , dt , par2 ) ; % An uniform scaled parameter set of N2 pts

% figure(1) ; clf ; gnr_chi_plot( 3.0 , M1 , dt , par1(5:9) )  ; % plot the gain as well as
% figure(2) ; clf ; gnr_chi_plot( 0.0 , M2 , dt , par2(5:9) )  ; % the saturable aborption


%% ------------------3-Spatially uniform IC for the two sections-----------

% Generate an initial condition, i.e. a mesh of M points in the past
% and N points in space. All variables are set to zero excepted the carrier 
% that are set around 3.0 in the gain section and 0 in the absorber. 
% We add a gaussian fluctuation of 1.e-03. 

MSH1 = gnr_uni_msh( M1 , N1 , 1.e-02 , 3. )  ;
MSH2 = gnr_uni_msh( M2 , N2 , 1.e-02 , 0. )  ;

%% ------------------4-Time integration -----------------------------------

% Time integration, over 2000 single trips, of two sections, saving data every dt

S = 2000*(N1+N2) ; V = 2 ; Samp = 1 ; 

D1 = 1 ; D2 = 1;  % no decimation at all, vanilla TWM with meshes of 513 and 17 for both sections

tic; [ tr_1_1  , MSH1a , MSH2a ] = freetwm( S , V , Samp , MSH1 , PAR1 , D1 , MSH2 , PAR2 , D2 , BC ) ; toc

D1 = 16 ; D2 = 2; % we use 1 point every 16 in the gain section, and 1 point every 2 in the absorber section 

tic; [ tr_16_2 , MSH1b , MSH2b ] = freetwm( S , V , Samp , MSH1 , PAR1 , D1 , MSH2 , PAR2 , D2 , BC ) ; toc

D1 = 32 ; D2 = 4; % we use 1 point every 32 in the gain section, and 1 point every 5 in the absorber section 

tic; [ tr_32_4 , MSH1c , MSH2c ] = freetwm( S , V , Samp , MSH1 , PAR1 , D1 , MSH2 , PAR2 , D2 , BC ) ; toc

D1 = 32 ; D2 = 8; % we use 1 point every 32 in the gain section, and 1 point every 8 in the absorber section 

tic; [ tr_32_8 , MSH1d , MSH2d ] = freetwm( S , V , Samp , MSH1 , PAR1 , D1 , MSH2 , PAR2 , D2 , BC ) ; toc

%% ------------------5-Illustration----------------------------------------

t = ( (1:S) - 1 )*dt/1.e-09; % time axis in ns

fig=figure(1);clf;set( fig , 'Position' , [0 0 1600 750] ) ;

tmin = 24.9 ; tmax = 25 ;
ymin = 0    ; ymax = 50 ;
var  = 2    ; sec  =  2 ;
% ymax=1.1*max (abs(tr_1_1(:,var,sec)).^2);

%----------------------

subplot(4,4,1:3); set(gca,'FontSize',22,'LineWidth',2);
plot( t , abs(tr_1_1(:,var,sec)).^2 , 'k' , 'LineWidth', 2 );
ylabel('I_+(L)'); axis([0 tmax ymin ymax]) ;

subplot(4,4,4); set(gca,'FontSize',22,'LineWidth',2);
plot( t , abs(tr_1_1(:,var,sec)).^2 , 'k' , 'LineWidth', 2 );
axis([tmin tmax ymin ymax]) ;

%----------------------

subplot(4,4,5:7); set(gca,'FontSize',22,'LineWidth',2);
plot( t , abs(tr_16_2(:,var,sec)).^2 , 'k' , 'LineWidth', 2 );
ylabel('I_+(L)'); axis([0 tmax ymin ymax]) ;

subplot(4,4,8); set(gca,'FontSize',22,'LineWidth',2);
plot( t , abs(tr_16_2(:,var,sec)).^2 , 'k' , 'LineWidth', 2 );
axis([tmin tmax ymin ymax]) ;
%----------------------

subplot(4,4,9:11); set(gca,'FontSize',22,'LineWidth',2);
plot( t , abs(tr_32_4(:,var,sec)).^2 , 'k' , 'LineWidth', 2 );
ylabel('I_+(L)'); axis([0 tmax ymin ymax]) ;

subplot(4,4,12); set(gca,'FontSize',22,'LineWidth',2);
plot( t , abs(tr_32_4(:,var,sec)).^2 , 'k' , 'LineWidth', 2 );
axis([tmin tmax ymin ymax]) ;
%----------------------

subplot(4,4,13:15); set(gca,'FontSize',22,'LineWidth',2);
plot( t , abs(tr_32_8(:,var,sec)).^2 , 'k' , 'LineWidth', 2 );
ylabel('I_+(L)'); xlabel('time (ns)');axis([0 tmax ymin ymax]) ;

subplot(4,4,16); set(gca,'FontSize',22,'LineWidth',2);
plot( t , abs(tr_32_8(:,var,sec)).^2 , 'k' , 'LineWidth', 2 );
xlabel('time (ns)'); axis([tmin tmax ymin ymax]) ;

% Mesh reconstruction from past value 

gnr_msh_plot( [MSH1a , MSH2a ]  , 2 , 'pt' ); % nice mesh witout decimation 

gnr_msh_plot( [MSH1d , MSH2d ]  , 3 , 'pt' ); % ugly mesh with strong decimation

gnr_msh_plot( [unfold(MSH1d,32,PAR1) , unfold(MSH2d,8,PAR2)] , 4 , 'pt' ); % nice mesh reconstructed from past value of the ugly mesh















