%-------------------------------------------------------------------------- 
% This script is supposed to be run only in order to get a cpu wall time.
% It represents a single section Fabry-Pérot laser with cleaved facets
% The cpu time is returned by the build-in function tic ; toc ;
%--------------------------------------------------------------------------

addpath('../System');

M   =  25 ;  % Number of point in the past
N   = 400 ;  % Number of point in space.

% par is a column vector that contains the values 
% of the 14 parameters given in inverse seconds

par = [3.0e10  0.e11 0.e11                        ... % waveguide
       1.6e11  8.e12 0.e12 90.e12 3.e11 3.e12 0.  ... % QW response
       150e08  1.e08 7.e08  1.e07 3.e11           ... % Carrier response
       1.e-02]';                                      % SP

tau_c = 12.5e-12 ;                      % cavity single trip (i.e. 40 GHz rep rate)
dt    = tau_c/N  ;                      % time step used in the scaling
PAR   = gnr_uni_param( N , dt , par ) ; % Uniform scaled parameter set of N points

% The boundary matrix corresponds to cleaved facets 
% Hence the amplitude reflectivity is sqrt(30%)

BC = zeros(2,2)  ; 
BC(1,1) = 0.55   ; 
BC(2,2) = 0.55   ; 
BC = complex(BC) ;      %  The BC matrix must be complex 
                                             
% we generate an initial condition, i.e. a mesh of M points in the past
% and N points in space. All variables are set to zero excepted
% the carrier that are set around 1.5 transparency. We add to all
% variables a gaussian fluctuation of variance 1.e-03.

MSH_IC = gnr_uni_msh( M , N , 1.e-03 , 1.5 )  ;

% We can start time integration of this single section, 
% over 100 single trips, saving data every 2*dt

S = 500*N ; V = 1 ;  Samp = 2 ;

tic; [ trace , MSH_FC ] = freetwm( S , V , Samp , MSH_IC , PAR , 1 , BC ) ; toc;

% ------------------------------------------------------------------------
