/*
----------------------------Authors -----------------------------------

Julien Javaloyes
Salvador Balle

Universitat de les Illes Baleares
Palma de Mallorca

-----------------------------Copyright --------------------------------

This file is part of Freetwm

Freetwm is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 3 of the License, or (at your
option) any later version.

Freetwm is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Freetwm; see the file COPYING.  
If not, see <http://www.gnu.org/licenses/>.

-----------------------------------------------------------------------
*/


/* ---------------------------------------------------------------------------------------------------- */
/* -----------------------Perform a circular shift of a section---------------------------------------- */
/* ---------------------------------------------------------------------------------------------------- */

void section_shift( section * msh ) 
{
  int m , M = msh->M ;

  dcvec tmp_Ep = msh->Ep[M-1] , tmp_Em = msh->Em[M-1] ;
  dcvec tmp_Pp = msh->Pp[M-1] , tmp_Pm = msh->Pm[M-1] ;

  dvec  tmp_D0 = msh->D0[M-1] ; 
  dcvec tmp_D2 = msh->D2[M-1] ;

#if APPROX==0 

  dcvec tmp_Dp2Em = msh->Dp2Em[M-1] ; 
  dcvec tmp_Dm2Ep = msh->Dm2Ep[M-1] ;

#endif

  for(m=M-1;m>0;m--)
    {
      msh->Ep[m] = msh->Ep[m-1] ; 
      msh->Em[m] = msh->Em[m-1] ; 
      msh->Pp[m] = msh->Pp[m-1] ; 
      msh->Pm[m] = msh->Pm[m-1] ; 
      msh->D0[m] = msh->D0[m-1] ; 
      msh->D2[m] = msh->D2[m-1] ;

#if APPROX==0 

      msh->Dp2Em[m] = msh->Dp2Em[m-1] ;
      msh->Dm2Ep[m] = msh->Dm2Ep[m-1] ;

#endif

    }

  msh->Ep[0] = tmp_Ep ; 
  msh->Em[0] = tmp_Em ;  
  msh->Pp[0] = tmp_Pp ; 
  msh->Pm[0] = tmp_Pm ; 
  msh->D0[0] = tmp_D0 ; 
  msh->D2[0] = tmp_D2 ;   

#if APPROX==0 
   
  msh->Dp2Em[0] = tmp_Dp2Em ;  
  msh->Dm2Ep[0] = tmp_Dm2Ep ; 

#endif
   
}


/* ---------------------------------------------------------------------------------------------------- */
/* -------------------------------The Actual TWM time-stepper------------------------------------------ */
/* ---------------------------------------------------------------------------------------------------- */

void Section_Update( section * msh , double mod_Om_gap , double mod_J ,  double mod_A )
{
  int j , M = msh->M , J = msh->J , Ne = msh->Ne ;

#pragma omp parallel for  /* everything after is local to each thread and not shared */
  for( j = 1 ; j <= Ne ; j++ )
    {
      int i , m ;

      int incl = (j == 1 ) ? 1 : J ;
      int incr = (j == Ne) ? 1 : J ;

      const REAL
	* hloss  = msh->Par[0]  , * hdbr_x = msh->Par[1]  , * hdbr_y = msh->Par[ 2] , 
	* hchi0  = msh->Par[3]  , * hgperp = msh->Par[4]  , * homgap = msh->Par[ 5] , * homtop = msh->Par[ 6] , * shb   = msh->Par[ 9] , 
	*     hJ = msh->Par[10] , *     hA = msh->Par[11] , *     hB = msh->Par[12] , *     hC = msh->Par[13] , * hD4q2 = msh->Par[14] ;
      
      REAL complex Ep_n  , Em_n  , Pp_n  , Pm_n   , D2_n   , hdbr_p_jm1  , hdbr_m_jp1 ;
      REAL complex buff1 , buff2 , buff3 , buffD0 , buffD2 , chi , dchi  ;
      REAL complex Kp , Km , Vp , Vm , a , b , c , d , k , k0 , overdet  ;
 
      REAL  D0_j , D0_n , hRD , hDRD , r , ID ;
     
      /* Solve : carrier update via Euler */
     
      D0_j = msh-> D0[0][j] ;

      hRD    = ( hA[j] + mod_A + ( hB[j] + hC[j] * D0_j ) * D0_j ) * D0_j ;  
    
      buffD0 =  msh-> Ep[0][j]  * conj(  msh-> Pp[0][j] )  +  msh-> Em[0][j]  * conj(  msh-> Pm[0][j] ) ;
      D0_n   = D0_j +  hJ[j] + mod_J - hRD  - 2.*SCALING_FACTOR*cimag(buffD0) ; 
	  
      /* Solve : grating update  via Euler */

      hDRD   =   hA[j] + mod_A + ( 2.*hB[j] + 3.*hC[j] * D0_j ) * D0_j          ;

      buffD2 =  msh-> Pp[0][j]  * conj(  msh-> Em[0][j] )  -  msh-> Ep[0][j]  * conj(  msh-> Pm[0][j] ) ;
      D2_n   =   msh->D2[0][j] * (1. -  hDRD - hD4q2[j])  - I * SCALING_FACTOR * buffD2 ;

      /* Form the 2x2 system to solve  : 2x2 matrix  */

      ID = D0_n / dD_LUT ; i = (int)ID ; r =  ID - (REAL)i  ;    /* find the value of the index ID & remainder [0,1] in LUT */
      if ( i > DIM_LUT - 2 ) 
	{ 
	  mexPrintf("\t\t\t\t --error-- right LUT index %i out of range\n",i) ;
 	  mexPrintf("\t\t\t\t --error-- Try to increase DIM_LUT in freetwm_flags.c or in the makefile and recompile \n",i) ; mexErrMsgTxt(""); 
	}

      chi  = (1-r) * msh-> chi[i][0] + r * msh-> chi[i+1][0]  ;  /* ---    linear interpolation    --- */
      dchi = (1-r) * msh->dchi[i][0] + r * msh->dchi[i+1][0]  ;  /* ---   between two LUT points   --- */

      buff1 =  chi / 2. * hchi0[j]  ; 
      buff2 = dchi / 2. * hchi0[j]  *       D2_n   ; 
      buff3 = dchi / 2. * hchi0[j]  * conj( D2_n ) ; 
 
      a  = 0.5 * incl * (  hloss[j] - I * buff1 ) + 1. ;
      d  = 0.5 * incr * (  hloss[j] - I * buff1 ) + 1. ;
      b  = 0.5 * incl * (  I * hdbr_x[j] - hdbr_y[j] - I * buff2 ) ;
      c  = 0.5 * incr * (  I * hdbr_x[j] + hdbr_y[j] - I * buff3 ) ;

      overdet = 1. / ( a*d - b*c ) ;

      /* convolution  */

      Kp = Km = Vp = Vm = 0. ; 
      k = k0 = cexp(  - I * ( homgap[j] + mod_Om_gap ) ) ; 

#if APPROX == 0

      for( m = 0 ; m < M ; m++ )  
	{
	  ID = msh->D0[m][j] / dD_LUT ;  i = (int)ID ; r =  ID - (REAL)i  ;      /* --- index ID & remainder [0,1] --- */   

	  if ( i > DIM_LUT - 2 ) 
	    { 
	      mexPrintf("\t\t\t\t --error-- right LUT index %i out of range\n",i) ;
	      mexPrintf("\t\t\t\t --error-- Try to increase DIM_LUT in freetwm_flags.c or in the makefile and recompile \n",i) ; mexErrMsgTxt(""); 
	    }

	  chi  =  k * ( (1-r) * msh-> chi[i][m+1] + r * msh-> chi[i+1][m+1] ) ;  /* ---    linear interpolation    --- */
	  dchi =  k * ( (1-r) * msh->dchi[i][m+1] + r * msh->dchi[i+1][m+1] ) ;  /* ---   between two LUT points   --- */

	  Kp  +=  msh->Ep[m][j] * chi + msh->Dp2Em[m][j] * dchi ; 
	  Km  +=  msh->Em[m][j] * chi + msh->Dm2Ep[m][j] * dchi ;

	  k   *= k0 ;                                                        /* --- update exp prefactor for next step-- */
	}
      Kp *= hchi0[j] ; Km *= hchi0[j] ;                                      /* --- We multiply by the chi0 prefactor -- */
  
#else

      for( m = 0 ; m < M ; m++ )  
	{
	  chi  =  k * ( (1-r) * msh-> chi[i][m+1] + r * msh-> chi[i+1][m+1] ) ;  /* ---    linear interpolation    --- */
	  dchi =  k * ( (1-r) * msh->dchi[i][m+1] + r * msh->dchi[i+1][m+1] ) ;  /* ---   between two LUT points   --- */

	  Kp  +=  msh->Ep[m][j] * chi  ; Km  += msh->Em[m][j] *  chi ;
	  Vp  +=  msh->Ep[m][j] * dchi ; Vm  += msh->Em[m][j] * dchi ; 

	  k   *= k0 ;                                                        /* --- update exp prefactor for next step -- */
	}
      Kp = hchi0[j] * ( Kp +      D2_n  * Vm ) ;                             /* --- We multiply by the chi0 prefactor  -- */
      Km = hchi0[j] * ( Km + conj(D2_n) * Vp ) ;                             /* --- and mix the cross coupling E+ & E- -- */

#endif
  
      /* Form the 2x2 system to solve  : vector   */

      hdbr_p_jm1 = hdbr_x[j-1] + I * hdbr_y[j-1] ;
      hdbr_m_jp1 = hdbr_x[j+1] - I * hdbr_y[j+1] ;

      Vp = ( 1. - 0.5*incl*hloss[j-1] ) *  msh->Ep[J-1][j-1] + (0.5*I*incl) * ( msh->Pp[J-1][j-1] + Kp - hdbr_p_jm1 * msh->Em[J-1][j-1] ) ;
      Vm = ( 1. - 0.5*incr*hloss[j+1] ) *  msh->Em[J-1][j+1] + (0.5*I*incr) * ( msh->Pm[J-1][j+1] + Km - hdbr_m_jp1 * msh->Ep[J-1][j+1] ) ;

      /* Solve : field update  */
	  
      Ep_n = ( d * Vp - b * Vm ) * overdet ;
      Em_n = ( a * Vm - c * Vp ) * overdet ;
	  
      /* Solve : pola update  */

      Pp_n = Kp + buff1 * Ep_n  + buff2 * Em_n ;
      Pm_n = Km + buff1 * Em_n  + buff3 * Ep_n ;
     
      /* Solve : carrier update via mid point */
	
      buffD0 = 0.5 * ( buffD0 +  Ep_n  * conj( Pp_n )  +  Em_n  * conj( Pm_n ) ) ; 
      D0_n  = D0_j + (  hJ[j] + mod_J - hRD  - 2.*SCALING_FACTOR*cimag(buffD0) ) / ( 1. + 0.5*hDRD ) ;
	  
      /* Solve : grating update via midpoint */

      buffD0 = 0.5 *( hDRD + hD4q2[j] )  ;
      buffD2 = 0.5 * ( buffD2 + Pp_n * conj(Em_n) - Ep_n * conj(Pm_n) ) ;
      D2_n   = ( msh->D2[0][j] * (1. - buffD0 )  - I * SCALING_FACTOR * buffD2 ) / ( 1. + buffD0 ) ;

      /* put theses new j-guys at the end of the m-array, before the permutation   */
 
      msh->Ep[M-1][j] = Ep_n ;  msh->Em[M-1][j] = Em_n ;  
      msh->Pp[M-1][j] = Pp_n ;  msh->Pm[M-1][j] = Pm_n ;  
      msh->D0[M-1][j] = D0_n ;  msh->D2[M-1][j] = D2_n ;

#if APPROX == 0
      
      /* Evaluate redondant variables used to speed up the Kernel Calculation      */

      msh->Dp2Em[M-1][j] =      D2_n * Em_n ; 
      msh->Dm2Ep[M-1][j] = conj(D2_n)* Ep_n ;
 
#endif
      
      /* -------------------------------------------------------------------------------*/
    } /* end of OpenMp for, sync barrier */
}



/* ---------------------------------------------------------------------------------------------------- */
/* --------------------------The passive section time-stepper------------------------------------------ */
/* ---------------------------------------------------------------------------------------------------- */

void Section_Update_Passive( section * msh )
{
  int j , M = msh->M , J = msh->J , Ne = msh->Ne ;

#pragma omp parallel for                         /* everything after is local to each thread and not shared */
  for( j = 1 ; j <= Ne ; j++ )
    {
      int incl = (j == 1 ) ? 1 : J ;
      int incr = (j == Ne) ? 1 : J ;

      const REAL * hloss  = msh->Par[0] , * hdbr_x = msh->Par[1] , * hdbr_y = msh->Par[ 2] ;
      
      REAL complex  Vp , Vm , a , b , c , d , overdet ; 
   
      /* Form the 2x2 system to solve  : 2x2 matrix  */

      a  = ( 0.5 * incl ) * ( hloss[j] ) + 1. ;
      d  = ( 0.5 * incr ) * ( hloss[j] ) + 1. ;
      b  = ( 0.5 * incl ) * ( I * hdbr_x[j] - hdbr_y[j] ) ;
      c  = ( 0.5 * incr ) * ( I * hdbr_x[j] + hdbr_y[j] ) ;

      overdet = 1. / ( a*d - b*c ) ;

      /* Form the 2x2 system to solve  : vector   */

      Vp = ( 1. - (0.5*incl) * hloss[j-1] ) * msh->Ep[J-1][j-1] - (0.5*I*incl) * ( hdbr_x[j-1] + I * hdbr_y[j-1] ) * msh->Em[J-1][j-1] ;
      Vm = ( 1. - (0.5*incr) * hloss[j+1] ) * msh->Em[J-1][j+1] - (0.5*I*incr) * ( hdbr_x[j+1] - I * hdbr_y[j+1] ) * msh->Ep[J-1][j+1] ;

      /* Solve : field update & put theses new j-guys at the end of the m-array, before the permutation */
	  
       msh->Ep[M-1][j] = ( d * Vp - b * Vm ) * overdet ;
       msh->Em[M-1][j] = ( a * Vm - c * Vp ) * overdet ;	  
    }                                           /* end of OpenMp for, sync barrier */  
}




/* ------------------------------------------------------------------------------------------------------------------------------------- */
/* ---------------------------------------------------- EOF----------------------------------------------------------------------------- */
/* ------------------------------------------------------------------------------------------------------------------------------------- */
