/*
----------------------------Authors -----------------------------------

Julien Javaloyes
Salvador Balle

Universitat de les Illes Baleares
Palma de Mallorca

-----------------------------Copyright --------------------------------

This file is part of Freetwm

Freetwm is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 3 of the License, or (at your
option) any later version.

Freetwm is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Freetwm; see the file COPYING.  
If not, see <http://www.gnu.org/licenses/>.

-----------------------------------------------------------------------
*/

/* ---------------------------------------------------------------------------------------------------- */
/* -------------------------------        Random part        ------------------------------------------ */
/* ---------------------------------------------------------------------------------------------------- */

#if HAVEMKL == 1

#include <mkl.h>
#include <mkl_vml.h>
#include <mkl_vsl.h>

/* ---------------------------------------------------------------------------------------------------- */
/* -------------------------------      The MKL section shake    -------------------------------------- */
/* ---------------------------------------------------------------------------------------------------- */

void Section_Shake_mkl( section * msh , VSLStreamStatePtr * stream)
{
  int j,j4,NN = msh->Ne ;
  float * noise = (float *) malloc( (unsigned) (4*NN) * sizeof(float) ) ;

  vsRngGaussian(  VSL_METHOD_SGAUSSIAN_ICDF , stream , 4*NN , noise , 0.0 , 1. ) ;

  for( j=0 ; j < NN ; j++ )
    {
      j4=4*j;

      msh->Pp[0][j+1] += msh->Par[15][j+1] * ( noise[ j4   ] + I * noise[ j4+1 ] ) ;
      msh->Pm[0][j+1] += msh->Par[15][j+1] * ( noise[ j4+2 ] + I * noise[ j4+3 ] ) ;
    }
  free(noise);   
}

#else 

/* ---------------------------------------------------------------------------------------------------- */
/* -------gcc srand() + Zigurat gaussian unit variance random number generator------------------------- */
/*----------------------- This gives the gnu section shaker ------------------------------------------- */
/* ---------------------------------------------------------------------------------------------------- */

REAL complex grn() 
{
  REAL r,v1,v2; 

  do 
    { 
      v1 = 2.*rand()/RAND_MAX-1.;
      v2 = 2.*rand()/RAND_MAX-1.;

      r = v1*v1 + v2*v2 ; 
    } 
  while ( r >= 1.0 );  

  return ( v1 + I * v2 ) * sqrt(-2.0*log(r)/r); 

} 

void Section_Shake_gcc( section * msh )
{ 
  int j ; 

  for( j = 1 ; j <= msh->Ne ; j++ )
    {
      msh->Pp[0][j] += msh->Par[15][j]*grn() ; 
      msh->Pm[0][j] += msh->Par[15][j]*grn() ; 
    } 
} 
	  
#endif



/* ------------------------------------------------------------------------------------------------------------------------------------- */
/* ---------------------------------------------------- EOF----------------------------------------------------------------------------- */
/* ------------------------------------------------------------------------------------------------------------------------------------- */
