/*
----------------------------Authors -----------------------------------

Julien Javaloyes
Salvador Balle

Universitat de les Illes Baleares
Palma de Mallorca

-----------------------------Copyright --------------------------------

This file is part of Freetwm

Freetwm is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 3 of the License, or (at your
option) any later version.

Freetwm is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Freetwm; see the file COPYING.  
If not, see <http://www.gnu.org/licenses/>.

-----------------------------------------------------------------------
*/

#define MAX( a , b ) (  (a) > (b) ? (a) : (b)   ) 
#define MIN( a , b ) (  (a) > (b) ? (b) : (a)   ) 

/* ---------------------------------------------------------------------------------------------------- */
/* -----------------Check if matlab input is a MxNx6 complex array and return the size----------------- */
/* ---------------------------------------------------------------------------------------------------- */


void Is_A_Mesh( int * M , int * N , const mxArray ** prhs , const int pos ) 
{
 const mwSize * arg_dim  =  mxGetDimensions( prhs[pos]  ) ;

 if (mxGetNumberOfDimensions( prhs[pos] ) != 3) { mexPrintf("\t\t\t\t --error-- var %i is not a 3dim array\n"   ,pos+1) ; mexErrMsgTxt(""); }
  if ( arg_dim[2] != 6)                         { mexPrintf("\t\t\t\t --error-- Third dim of arg %i must be 6\n",pos+1) ; mexErrMsgTxt(""); }
  if ( !mxIsComplex( prhs[pos] ) )              { mexPrintf("\t\t\t\t --error-- arg %i must be complex\n"       ,pos+1) ; mexErrMsgTxt(""); }
  
  *M = arg_dim[0] ;  *N = arg_dim[1] ;
}

/* ---------------------------------------------------------------------------------------------------- */
/* --------Check if matlab input is a LxC real or complex array with the good values ------------------ */
/* ---------------------------------------------------------------------------------------------------- */

bool Is_A_Matlab_Matrix( const char * type , const char * abort_condition , const int L , const int C ,  const mxArray ** prhs , const int pos ) 
{
  bool return_value = true , abort_on_error ;  const mwSize *  arg_dim = mxGetDimensions( prhs[pos] ) ; 

  if( strcmp( abort_condition , "abort_on_failure"  ) == 0 )  
    abort_on_error = true ; 
  else 
    {
      if( strcmp( abort_condition , "no_abort_on_failure"  ) == 0 )  
	abort_on_error = false ; 
      else 	
	{ 
	  mexPrintf("\t\t\t\t --error-- abort condition in Is_A_Matlab_Matrix not recognized")   ; 
	  mexErrMsgTxt(""); 
	}
    }

 if (mxGetNumberOfDimensions( prhs[pos] ) != 2 ) 
   { 
     return_value = false ;
     if(abort_on_error)  
       { 
	 mexPrintf("\t\t\t\t --error-- var %i is not a 2dim array\n" ,pos+1)  ; 
	 mexErrMsgTxt(""); 
       } 
   }
 
 if ( arg_dim[0] != L )  
   {  
     return_value = false ;  
     if(abort_on_error)  
       { 
	 mexPrintf("\t\t\t\t --error-- line nb of var %i is not %i\n",pos+1,L) ; 
	 mexErrMsgTxt(""); 
       }
   }

 if ( arg_dim[1] != C )   
   {  
     return_value = false ;  
     if(abort_on_error)  
       { 
	 mexPrintf("\t\t\t\t --error-- cols nb of var %i is not %i\n",pos+1,C) ; 
	 mexErrMsgTxt(""); 
       }
   }
 
 if( strcmp( type , "real" ) == 0 )
   { 
     if ( mxIsComplex( prhs[pos] ) )  
       {   
	 return_value = false ;  
	 if(abort_on_error)  
	   { 
	     mexPrintf("\t\t\t\t --error-- arg %i must not be complex \n",pos+1)   ; 
	     mexErrMsgTxt(""); 
	   }
       }
   } 

   
 if( strcmp( type , "complex" ) == 0 )
   {  
     if ( !mxIsComplex(prhs[pos] ) )    
       { 	 
	 return_value = false ;  
	 if(abort_on_error)  
	   { 
	     mexPrintf("\t\t\t\t --error-- arg %i must be complex\n",pos+1)   ; 
	     mexErrMsgTxt(""); 
	   }
       } 
   }
  
if( strcmp( type , "real" ) != 0 &&  strcmp( type , "complex" ) != 0 )
  {  
    return_value = false ;  
    mexPrintf("\t\t\t\t --error-- arg %i type not found",pos+1)   ; 
    mexErrMsgTxt(""); 
  } 
   
 return return_value ;
}

/* ---------------------------------------------------------------------------------------------------- */
/* --------------------Read matlab input and copy data in a section mesh------------------------------- */
/* ---------------------------------------------------------------------------------------------------- */

void  Initialize_Section_IC(  section * sim , const mxArray * prhs )
{

  int j , m , n , ne , N = sim->N , M = sim->M , J = sim->J , Ne = sim->Ne;

  double *  ptr1_x = mxGetPr(prhs) , * ptr1_y = mxGetPi(prhs) ;
      
  for( m = 0 ; m < M ; m++ )
    { 	
      for( n = 1 ; n <= Ne ; n++ )  /* matlab C array start at 0 but we fill the mesh at 1*/
	{ 
	  ne = (n-1)*J ;  //printf("in mesh n=%i\t ne=%i\n",n,ne);
	  
	  j = M*ne + m + 0*M*N ; sim-> Ep[m][n] = ptr1_x[j] + I * ptr1_y[j] ;    
	  j = M*ne + m + 1*M*N ; sim-> Em[m][n] = ptr1_x[j] + I * ptr1_y[j] ;   
	  j = M*ne + m + 2*M*N ; sim-> Pp[m][n] = ptr1_x[j] + I * ptr1_y[j] ;    
	  j = M*ne + m + 3*M*N ; sim-> Pm[m][n] = ptr1_x[j] + I * ptr1_y[j] ;  
	  j = M*ne + m + 4*M*N ; sim-> D0[m][n] = ptr1_x[j]                 ;   /* Im(D0) is ignored */    
	  j = M*ne + m + 5*M*N ; sim-> D2[m][n] = ptr1_x[j] + I * ptr1_y[j] ;  

#if APPROX == 0

	  /* Initialize the redondant arrays used to speed the convolution Kernel evaluation */

	  sim-> Dp2Em[m][n] =      sim-> D2[m][n]  * sim-> Em[m][n];
	  sim-> Dm2Ep[m][n] = conj(sim-> D2[m][n]) * sim-> Ep[m][n];
#endif	    
	}
    }
}

/* ---------------------------------------------------------------------------------------------------- */
/* --------------------Read matlab input and copy data in a section param------------------------------ */
/* ---------------------------------------------------------------------------------------------------- */

void  Initialize_Section_PAR(  section * sim , const mxArray * prhs )
{
  int j , p , n , ne , N = sim->N  , J = sim->J , Ne = sim->Ne;
  
  double  * ptr1_x  = mxGetPr(prhs) ; 
  
  for( p = 0 ; p < PAR_NUM ; p++ ) 
    for( n = 1 ; n <= Ne ; n++ )  /* matlab C array start at 0 but we fill the mesh at 1 */
      { 
	ne = (n-1)*J ; // printf("in par n=%i\t ne=%i\n",n,ne);
	
	j = PAR_NUM*ne + p  ; sim->Par[p][n] =ptr1_x[j] ;  /* Fill the parameter matrix  */
      }

  /* Check if the section is passive i.e. chi0[1..N] is zero */

  double test_chi = 0. ; for( n = 1 ; n <= Ne ; n++ ) test_chi += sim->Par[3][n] ; 
 
  sim->Passive = (test_chi == 0 ) ? true : false ;
  
}

/* ------------------------------------------------------------------------------------------------------------------------------------- */
/* ---------------------------------------------------- EOF----------------------------------------------------------------------------- */
/* ------------------------------------------------------------------------------------------------------------------------------------- */

double Fermi    ( double w ,  double gma_cv  , double  BF ){  return 1. / ( 1. + exp(  w / gma_cv  - BF ) )  ; }

double Fermi_inv( double y ,  double gma_cv  , double  BF ){  return ( BF + log( 1./y - 1. ) ) * gma_cv   ; }

void gnr_chi_time_kernel( double complex * chi_t , double complex * dchi_t_dD , double D0 , int M , double  gma , double om_t , double gma_c , double gma_v ) 
{
  int m , Nc , Nv , nc , nv ; double df , dom_max , BF, dBFdD , w1 , w2 , z ;

  double  BFc    = log( ( exp( D0*gma/gma_c ) - 1. ) ) ;                                  /* large band approx */
  double  BFv    = log( ( exp( D0*gma/gma_v ) - 1. ) ) ;                                  /* large band approx */
  double  dBFcdD = (gma/gma_c) * exp( D0*gma/gma_c ) / ( exp( D0*gma/gma_c ) - 1. ) ;     /* large band approx */
  double  dBFvdD = (gma/gma_v) * exp( D0*gma/gma_v ) / ( exp( D0*gma/gma_v ) - 1. ) ;     /* large band approx */

  double * Fc , * Fv  , * Wc , * Wv  ; double Fc_mid , Fv_mid , s ;
 
  Fc =  malloc( sizeof(double) ) ;   Wc =  malloc( sizeof(double) ) ; 
  Fv =  malloc( sizeof(double) ) ;   Wv =  malloc( sizeof(double) ) ; 

  Wc[0] = 0. ;   Fc[0] = Fermi( 0. , gma_c  ,  BFc ) ;
  Wv[0] = 0. ;   Fv[0] = Fermi( 0. , gma_c  ,  BFv ) ;
 
  dom_max = delta_quad *  om_t ;  Nc = Nv = 1 ;

  df = delta_quad * Fc[0] ;

  while( fabs ( Wc[Nc-1] - om_t ) > 1.e-6 )
    {
      z = Fc[Nc-1] - df ;  w1 =  z < 0. ? om_t : Fermi_inv( z , gma_c , BFc ) ; w2 = Wc[Nc-1] + dom_max ;
      Nc++ ; Fc = realloc( Fc , sizeof(double) * Nc ) ; Wc = realloc( Wc , sizeof(double) * Nc ) ;
      Wc[Nc-1] = MIN( w1 , w2 ) ; Fc[Nc-1] =  Fermi( Wc[Nc-1] , gma_c , BFc ) ;
    }
 
  df = delta_quad * Fv[0] ;

  while(fabs ( Wv[Nv-1] - om_t ) > 1.e-6 )
    {
      z = Fv[Nv-1] - df ;  w1 = z < 0. ? om_t : Fermi_inv( z , gma_v , BFv ) ; w2 = Wv[Nv-1] + dom_max ;
      Nv++ ; Fv = realloc( Fv , sizeof(double) * Nv ) ; Wv = realloc( Wv , sizeof(double) * Nv ) ;
      Wv[Nv-1] = MIN( w1 , w2 ) ; Fv[Nv-1] =  Fermi( Wv[Nv-1] , gma_v , BFv ) ;
    }
  
  for( m = 0 ; m < M ; m ++ )
    {
      s = m + 1.e-3 ;
      
      chi_t[m] = dchi_t_dD[m] = 0. ;

      for( nc = 0 ; nc < Nc - 1 ; nc++ )                   /* add the  contribution from the electrons over the band  */
      	{
      	  Fc_mid       = ( Fc[ nc + 1 ] + Fc[ nc ] ) * 0.5 ;
      	  chi_t[m]    += ( cexp(- I * Wc[ nc + 1 ] * s ) - cexp(- I * Wc[ nc ] * s ) ) * Fc_mid ;
      	  dchi_t_dD[m]+= ( cexp(- I * Wc[ nc + 1 ] * s ) - cexp(- I * Wc[ nc ] * s ) ) * Fc_mid * ( 1. - Fc_mid ) * dBFcdD ;
      	}
      
      for( nv = 0 ; nv < Nv - 1 ; nv++ )                   /* add the  contribution from the holes over the band      */
      	{
      	  Fv_mid       = ( Fv[ nv + 1 ] + Fv[ nv ] ) * 0.5 ;
      	  chi_t[m]    += ( cexp(- I * Wv[ nv + 1 ] * s ) - cexp(- I * Wv[ nv ] * s ) ) * Fv_mid ;
      	  dchi_t_dD[m]+= ( cexp(- I * Wv[ nv + 1 ] * s ) - cexp(- I * Wv[ nv ] * s ) ) * Fv_mid * ( 1. - Fv_mid ) * dBFvdD ;
	
      	}
      chi_t[m]     -=  cexp( - I * om_t * s ) - 1.      ;  /* add the carrier independent background contribution     */

      chi_t[m]     *=  exp( - gma * s ) /( - I * s )    ;  /* there is no bandgap here as it will varying             */
      dchi_t_dD[m] *=  exp( - gma * s ) /( - I * s )    ;  /* there is no bandgap here as it will varying             */

      chi_t[m]     *= -I / M_PI ;                          /* I from bloch eq , pi from normalization                 */
      dchi_t_dD[m] *= -I / M_PI ;                          /* I from bloch eq , pi from normalization                 */ 
    } 

free( Fc ) ; free( Fv ) ; 
free( Wc ) ; free( Wv ) ; 

}

/* ---------------------------------------------------------------------------------------------------- */
/* ----------------------- Evaluate the lookup table for each section --------------------------------- */
/* ----------------------        with a non zero temperature ---------------------- ------------------- */
/* ---------------------------------------------------------------------------------------------------- */


void Initialize_Section_LUT(  section * sim  )
{

  REAL complex r ; REAL D_i  ; int  i , m ;

  const REAL * hgperp = sim->Par[4]  , * homgap = sim->Par[5] , * homtop = sim->Par[6] , * hgma_c = sim->Par[7] , * hgma_v = sim->Par[8] ;

  if ( sim->Passive == true )  return ;

           /* we take the values of gamma, Omega_top,  gma_c , gma_v         */
           /* at the first spatial point, i.e. constant within the section   */
           /* at this point, only chi0 and omega_gap are still spatial funcs */

  REAL  hgma = hgperp[1] , hgma_cc =hgma_c[1] , hgma_vv = hgma_v[1] ;

  if (hgma_cc < hgma/100. ) { hgma_cc = hgma/100. ;  if(DBG>0) mexPrintf("--Minimal temperature of gma/100 reached for the electrons --\n") ; }
  if (hgma_vv < hgma/100. ) { hgma_vv = hgma/100. ;  if(DBG>0) mexPrintf("--Minimal temperature of gma/100 reached for the holes     --\n") ; }

#pragma omp parallel for  
  for( i = 0 ; i < DIM_LUT ; i++ ) 
    gnr_chi_time_kernel( sim-> chi[i] , sim-> dchi[i] , 1.e-06 + i*dD_LUT , sim->M + 1 , hgma , homtop[1], hgma_cc , hgma_vv ); /* D = 0 means Fermi level = -inf */

}
    
