function [ trace , MESHo ] = freetwm( K , V , S , MESHi , PARi , D_1 , BC , Inj ) 
%
%
%  [ trace , Mesh_1o ,..., Mesh_Vo ] = FREETWM( K , V , S , Mesh_1i , Par_1 , D_1,..., Mesh_Vi , Par_V , D_V , BC , Inj ) ;
% 
%        K: Number of integration steps 
%
%        V: Number of sections
%
%        S: Sample the output every S step(s).
%
% Each section has to contain N(v) spatial points and M(v) past values (for v=1..V)
%
% Mesh _1i, ..., Mesh_Vi : Initial condition complex profiles of size [ M(v) N(v) 6]  (for v=1..V)
%
% Mesh _1o, ..., Mesh_Vo :   Final condition complex profiles of size [ M(v) N(v) 6]  (for v=1..V)
%
%    Par_1, ..., Par_V   :   Real parameters spatial profiles of size [ 16 N(v) ]     (for v=1..V) 
%
%      D_1, ...,   D_V   :   Decimation factor used in the recasting into Delay eqs   (for v=1..V) 
%
%       BC: Complex boundary condition matrix of size [2V 2V]
%
%      Inj: Time dependent complex optical injection applied in the forward (resp. backward) 
%           direction of the first (resp. last) section. Hence size( Inj ) = [ K 2 ].
% 
%    trace: Field outputs at the first (resp. last) point in the forward (resp. backward) 
%           directions for each section. 
%           Carrier density at the first (resp. last) point for each section.
%           Carrier grating at the first (resp. last) point for each section.
%           Hence size(trace) = [ ceil(K/S) 6 V ]
end

%
% This is a dummy header giving only the error message
% The real function is the mex file with the same name
% that has to be compiled from the c file, see the manual
% for more informations.
%

% ----------------------------Authors -----------------------------------
% 
% Julien Javaloyes
% Antonio Pérez-Serrano
% Salvador Balle
% 
% Universitat de les Illes Baleares
% Palma de Mallorca
% 
% -----------------------------Copyright --------------------------------
% 
% This file is part of Freetwm
% 
% Freetwm is free software; you can redistribute it and/or modify it
% under the terms of the GNU General Public License as published by the
% Free Software Foundation; either version 3 of the License, or (at your
% option) any later version.
% 
% Freetwm is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
% FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
% for more details.
% 
% You should have received a copy of the GNU General Public License
% along with Freetwm; see the file COPYING.  
% If not, see <http://www.gnu.org/licenses/>.
% 
% -----------------------------------------------------------------------
