function [ fig ] = gnr_msh_plot( msh , Nfig , flag )
% --- gnr_msh_plot( msh , Nfig , flag )
%     make a nice plot of the profiles on figure nb Nfig;
%     if the flag is set to 'pt' the point number is used on
%     the abscissa, otherwise it is scaled between 0 and 1.

dm = size(msh); 
N  = dm(2);
if( nargin==3 && strcmp(flag,'pt')==1),
    z  = 1:N;
    mylim = [1 N];
    myxlabel='index';
else
    z  = linspace(0.5/N,1-0.5/N,N);
    mylim = [0 1];
    myxlabel='z/L';
end

%-------------------------------------------------------------------------%

fig=figure(Nfig); clf; set( fig , 'Position' , [0 0 1200 750] ) ;
WX = 0.4 ; HY1 = 0.41 ; HY2 = 0.41 ; DX = 0.055 ; DY = 0.085 ;


prog = ver('OCTAVE'); 

if( numel(prog) == 1 ),
    
    OCTAVE = 1 ;
    
    vec_I  = 221 ; vec_E  = 222 ;
    vec_D0 = 223 ; vec_D2 = 224 ;
       
else
    
    OCTAVE = 0 ;
    
    vec_D0 = [1.5*DX  DY WX HY1 ] ;  vec_I = [ 1.5*DX HY1+1.5*DY WX HY2 ] ;
    vec_D2 = [WX+3*DX DY WX HY1 ] ;  vec_E = [WX+3*DX 1.5*DY+HY1 WX HY2 ] ;
    
end



%-------------------------------------------------------------------------%


if(OCTAVE==1), subplot(vec_I); else subplot('Position',vec_I); end 

set(gca,'FontSize',22,'LineWidth',2);

hold on;
plot( z , abs(msh(1,:,1)).^2, '-r', z  , abs(msh(1,:,2)).^2 , '-b' , z  , abs(msh(1,:,1)).^2 + abs(msh(1,:,2)).^2 , '--k' ,  'LineWidth', 1) ; 
plot( z , abs(msh(1,:,1)).^2, '-r', z  , abs(msh(1,:,2)).^2 , '-b' ,  'LineWidth', 2) ; 

ylabel('\color{red} I_{+}(z) \color{black} & \color{blue} I_{-}(z)'); 
axis tight ; xlim(mylim); grid on ; box on ; set(gca,'XTickLabel',{''}) ;

 
%-------------------------------------------------------------------------%

if(OCTAVE==1), subplot(vec_E); else subplot('Position',vec_E); end 

set(gca,'FontSize',22,'LineWidth',2);

hold on;
plot( z , real( msh(1,:,1) ), '-r', z , imag( msh(1,:,1) ) , '-.r' , 'LineWidth', 2) ; 
plot( z , real( msh(1,:,2) ), '-b', z , imag( msh(1,:,2) ) , '-.b' , 'LineWidth', 2) ; 
hold off;

ylabel('\color{red} E_{+}(z) \color{black} & \color{blue} E_{-}(z)'); 

axis tight ; xlim(mylim) ; grid on ; box on ; set(gca,'XTickLabel',{''}) ;

 
%-------------------------------------------------------------------------%

if(OCTAVE==1), subplot(vec_D0); else subplot('Position',vec_D0); end 

set(gca,'FontSize',22,'LineWidth',2);

hold on;
plot( z , real(msh(1,:,5)), '-k', 'LineWidth', 2 ) ; 

ylabel('D_0(z)'); axis tight; xlim(mylim); xlabel(myxlabel); grid on ; box on ;

%-------------------------------------------------------------------------%

if(OCTAVE==1), subplot(vec_D2); else subplot('Position',vec_D2); end 

set(gca,'FontSize',22,'LineWidth',2);
hold on;
plot( z , 1.e02*real( msh(1,:,6) ), '-k', z  , 1.e02*imag( msh(1,:,6) ) , '-.k' , 'LineWidth', 2) ; 

ylabel('D_2(z) ( x 100 )'); axis tight; xlim(mylim); xlabel(myxlabel); grid on ; box on ;

%-------------------------------------------------------------------------%
end