function [ msh ] = gnr_uni_msh( M , N , sigma , Deq )
% ---  Generate a mesh object [ msh ] = gnr_uni_msh( M , N , sigma , Deq )
%
% M     : length of the past memory segment.
% N     : number of point in the spatial discretization.
% sigma : Amplitude of the gaussian deviation fluctuation
% Deq   : Equilibrium value of the carrier ( transparency at T=0 is D=1 )
%
% ---  All variables are centered around zero excepted
% ---  for the carrier D0 that are centered around Deq 

msh=zeros(M,N,6);

%IC for the field 

msh(:,:,1:2) = sigma * ( randn(M,N,2) + 1i*randn(M,N,2)   ) ;  %IC for the field amplitudes
msh(:,:,3:4) = sigma * ( randn(M,N,2) + 1i*randn(M,N,2)   ) ;  %IC for the polarisations
msh(:,:,5  ) = sigma *   randn(M,N,1) + Deq                 ;  %IC for the carrier density
msh(:,:,6  ) = sigma * ( randn(M,N,1) + 1i*randn(M,N,1)   ) ;  %IC for the lambda/2 grating


msh(:,:,5  ) = msh(:,:,5  ) .* ( msh(:,:,5  ) > 0 ) ;          % ensure that D >= 0

if( isreal(msh) ), msh=complex(msh); end

end
