function [ PAR ] = gnr_uni_param( N , dt , x )   
% --- PAR = gnr_uni_param( N , dt , x )
%    
%    Generates a spatially uniform scaled parameter set of N pts from a vector x(1:16). 
%    Some of the elements of x (*) are given in inverse seconds and the output  
%    parameters are scaled, when appropriate (*), by the time step dt, given in second.
%    
%
%  Waveguide parameters      : 
%    1 -  losses    (*)
%    2 -  Real(dfb) (*) 
%    3 -  Imag(dfb) (*) 
%
%  Active material parameters :
%    4 -  gain      (*) 
%    5 -  bandwidth (*)
%    6 -  bandgap   (*)
%    7 -  topband   (*)
%    8 -  gma_c     (*)   
%    9 -  gma_v     (*)    
%   10 -  Sat intensity of the medium
%
%  Carrier parameters        : 
%    11 -  Current  (*)
%    12-  A         (*)
%    13 - B         (*)
%    14 - C         (*)
%    15 - 4Dq^2     (*)
%    16 - Spontaneous Emission rate 

PAR = x(:,ones(N,1) );
    
tobescaled = [1:9 11:15] ;

PAR(tobescaled,:) = PAR(tobescaled,:)*dt ;

end

